@@ -1,4 +1,14 @@
-#!/usr/local/bin/python3.7
+#!/usr/local/bin/python3.8
+
+##===--- iwyu_tool.py -----------------------------------------------------===##
+#
+#                     The LLVM Compiler Infrastructure
+#
+# This file is distributed under the University of Illinois Open Source
+# License. See LICENSE.TXT for details.
+#
+##===----------------------------------------------------------------------===##
+
 """ Driver to consume a Clang compilation database and invoke IWYU.
 
 Example usage with CMake:
@@ -290,24 +300,19 @@ class Invocation(object):
     def start(self, verbose):
         """ Run invocation and collect output. """
         if verbose:
-            print('# %s' % self)
+            print('# %s' % self, file=sys.stderr)
 
         return Process.start(self)
 
 
-def parse_compilation_db(compilation_db_path):
-    """ Parse JSON compilation database and return a canonicalized form. """
-    if os.path.isdir(compilation_db_path):
-        compilation_db_path = os.path.join(compilation_db_path,
-                                           'compile_commands.json')
-
-    # Read compilation db from disk.
-    compilation_db_path = os.path.realpath(compilation_db_path)
-    with open(compilation_db_path, 'r') as fileobj:
-        compilation_db = json.load(fileobj)
-
-    # Expand symlinks.
+def fixup_compilation_db(compilation_db):
+    """ Canonicalize paths in JSON compilation database. """
     for entry in compilation_db:
+        # Convert relative paths to absolute ones if possible, based on the entry's directory.
+        if 'directory' in entry and not os.path.isabs(entry['file']):
+            entry['file'] = os.path.join(entry['directory'], entry['file'])
+
+        # Expand relative paths and symlinks
         entry['file'] = os.path.realpath(entry['file'])
 
     return compilation_db
@@ -324,12 +329,13 @@ def slice_compilation_db(compilation_db, selection):
     new_db = []
     for path in selection:
         if not os.path.exists(path):
-            print('WARNING: \'%s\' not found on disk.' % path)
+            print('warning: \'%s\' not found on disk.' % path, file=sys.stderr)
             continue
 
         found = [e for e in compilation_db if is_subpath_of(e['file'], path)]
         if not found:
-            print('WARNING: \'%s\' not found in compilation database.' % path)
+            print('warning: \'%s\' not found in compilation database.' % path,
+                  file=sys.stderr)
             continue
 
         new_db.extend(found)
@@ -365,12 +371,26 @@ def main(compilation_db_path, source_files, verbose, formatter, jobs,
          extra_args):
     """ Entry point. """
 
-    try:
-        compilation_db = parse_compilation_db(compilation_db_path)
-    except IOError as why:
-        print('Failed to parse JSON compilation database: %s' % why)
+    if not IWYU_EXECUTABLE:
+        print('error: include-what-you-use executable not found',
+              file=sys.stderr)
         return 1
 
+    try:
+        if os.path.isdir(compilation_db_path):
+            compilation_db_path = os.path.join(compilation_db_path,
+                                               'compile_commands.json')
+
+        # Read compilation db from disk.
+        compilation_db_path = os.path.realpath(compilation_db_path)
+        with open(compilation_db_path, 'r') as fileobj:
+            compilation_db = json.load(fileobj)
+    except IOError as why:
+        print('error: failed to parse compilation database: %s' % why,
+              file=sys.stderr)
+        return 1
+
+    compilation_db = fixup_compilation_db(compilation_db)
     compilation_db = slice_compilation_db(compilation_db, source_files)
 
     # Transform compilation db entries into a list of IWYU invocations.
